<?php

/**
 * @file
 * Saveva Language Menu functions that are only called on the admin page.
 */

/**
 * Overriding system settings form.
 */
function saveva_language_menu_system_settings_form($form, $automatic_defaults = TRUE) {
  $form['actions']['#type'] = 'container';
  $form['actions']['#attributes']['class'][] = 'form-actions';
  $form['actions']['#weight'] = 100;
  $form['actions']['submit'] = array('#type' => 'submit', '#value' => t('Save configuration'));

  if ($automatic_defaults) {
    $form = _system_settings_form_automatic_defaults($form);
  }

  // Adding our custom submission handler.
  $form['#submit'][] = 'saveva_language_menu_admin_settings_submit';

  $form['#submit'][] = 'system_settings_form_submit';
  // By default, render the form using theme_system_settings_form().
  if (!isset($form['#theme'])) {
    $form['#theme'] = 'system_settings_form';
  }
  return $form;
}

/**
 * Configure the administration form arguments.
 *
 * @see saveva_language_menu_menu()
 */
function saveva_language_menu_admin_settings() {
  $form = $menu_default = $language_list = $languages_enabled = $fallback_options = array();
  // Integration with the Domain Access module.
  if (module_exists('domain_conf')) {
    $main = variable_get('menu_main_links_source');
    $secondary = variable_get('menu_secondary_links_source');
    if ($main) {
      $menu_default['domainaccessmenu1st-' . $main . ':0'] = t('Domain Access: Main links');
    }
    if ($secondary) {
      $menu_default['domainaccessmenu2nd-' . $secondary . ':0'] = t('Domain Access: Secondary links');
    }
  }
  $language_list = language_list();
  foreach ($language_list as $lang => $langobj) {
    if ($langobj->enabled) {
      $languages_enabled[$lang] = $langobj->native;
    }
  }
  $custom_menus = menu_get_menus(FALSE);
  $menu_default += array('<nolangmenu>:0' => t('<nolangmenu> (Removes language items)'));
  $menu_default += menu_parent_options(array('main-menu' => 'Main Menu', 'user-menu' => 'User Menu'), array('mlid' => 0));

  $menu_default += menu_parent_options($custom_menus, array('mlid' => 0));

  $form['slm1-help'] = array(
    '#type' => 'fieldset',
    '#title' => t('Saveva language menu help'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

  $form['slm1-help']['help'] = array(
    '#type' => 'item',
    '#title' => '',
    '#markup' => t('You can add up to @menunumber language sets to menus. A set requires at least two activated languages out of the available site languages under <strong>Enable languages</strong>.
The language options can be added as childs. In this case you have to specify a parent title. The parent title has to be translated using the !translate.
<br /><br /><hr /><br />
The language options do not need translation if you use the native names, e.g. Deutsch for German, Español for Spanish. That\'s the recommended default behaviour, it saves you a lot of configuration work.
<br /><br /><hr /><br />
You can attach more than one language set to a menu and tune the visibility using the path inclusion and exclusion option to avoid the simultaneous display of language sets while being able to show more or less languages based on your specific needs,
e.g. a contact page may show all available languanges while other pages like the front page can be restricted to fewer languages. (Advanced users: If you need more than 4 language sets you may adapt the SAVEVA_LANGUAGE_MENUS constant in the module file.)
<br /><br /><hr /><br />
If you want to delete a language set from a menu, please select the <strong>&lt;nolangmenu&gt;</strong> option in the corresponding menu selector.
<br /><br /><hr /><br />
The link path for the language menu items updates automatically to reflect the loaded context, e.g. the front page switches to front page in different language, a content page to the language equivalent, if it exists. 
For this reason it is important not to change the path in the !menu.
<br /><br /><hr /><br />
If you use a parent for you language items you may need to ensure that the <strong>@showexpanded</strong> option is enabled in the !menu. The expanded options is enabled by default for the language items\' parent.
<br /><br /><hr /><br />
You can use the !menu to establish the correct order of your language menu items within the menu.
<br /><br /><hr /><br />
<strong>Important</strong>: The plugin will delete your language menu items from the menu if you select less than two languages under <strong>Enable languages</strong>. To delete all Saveva language menu items from your menus
you have to select <strong>&lt;nolangmenu&gt;</strong> in all <strong>Menu selectors</strong>. Uninstall will delete all language sets as well.
',
    array(
      '!translate' => l(t('String translation'),
      'admin/config/regional/translate/translate'),
      '!menu' => l(t('Menu management'),
      'admin/structure/menu'),
      '@showexpanded' => t('Show as expanded'),
      '@menunumber' => SAVEVA_LANGUAGE_MENUS,
    )),
  );

  $options = array(
    SAVEVA_LANGUAGE_MENU_VISIBILITY_NOTLISTED => t('All pages except those listed'),
    SAVEVA_LANGUAGE_MENU_VISIBILITY_LISTED => t('Only the listed pages'),
  );
  $title = t('Pages');

  $description = t("Specify pages by using their paths. Enter one path per line. The '*' character is a wildcard. Example paths are %blog for the blog page and %blog-wildcard for every personal blog. %front is the front page.",
  array(
    '%blog' => 'blog',
    '%blog-wildcard' => 'blog/*',
    '%front' => '<front>',
  ));

  for ($slm = 0; $slm < SAVEVA_LANGUAGE_MENUS; $slm++) {
    $fieldset = 'slm' . $slm . '-settings';
    $form[$fieldset] = array(
      '#type' => 'fieldset',
      '#title' => t('Saveva language menu @lmnum settings', array('@lmnum' => ($slm + 1))),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $form[$fieldset]['saveva_language_menu_menu_' . $slm] = array(
      '#type' => 'select',
      '#title' => t('Menu Selector'),
      '#description' => t('Menu to add the language switches. <em>(Default: %nolang)</em>', array('%nolang' => '&lt;nonlangmenu&gt;')),
      '#default_value' => variable_get('saveva_language_menu_menu_' . $slm, '<nolangmenu>:0'),
      '#options' => $menu_default,
    );

    $form[$fieldset]['saveva_language_menu_title_' . $slm] = array(
      '#type' => 'textfield',
      '#title' => t('Parent for language menu'),
      '#description' => t('Add a parent or leave blank to add language options without a dedicated parent.'),
      '#default_value' => variable_get('saveva_language_menu_title_' . $slm, 'Language'),
    );
    $form[$fieldset]['saveva_language_menu_languages_' . $slm] = array(
      '#title' => t('Enable languages'),
      '#type' => 'checkboxes',
      '#description' => t('Select the languages you want to make available in the menu.'),
      '#options' => $languages_enabled,
      '#default_value' => variable_get('saveva_language_menu_languages_' . $slm, array()),
    );
    $form[$fieldset]['saveva_language_menu_nativename_' . $slm] = array(
      '#type' => 'checkbox',
      '#title' => t('Use native name of languages in menus.'),
      '#description' => t('Use native names of the languages inside menus'),
      '#default_value' => variable_get('saveva_language_menu_nativename_' . $slm, 1),
    );
    $form[$fieldset]['saveva_language_menu_path_visibility_' . $slm] = array(
      '#type' => 'radios',
      '#title' => t('Show language menu on specific pages'),
      '#options' => $options,
      '#default_value' => variable_get('saveva_language_menu_path_visibility_' . $slm, SAVEVA_LANGUAGE_MENU_VISIBILITY_NOTLISTED),
    );
    $form[$fieldset]['saveva_language_menu_path_list_' . $slm] = array(
      '#type' => 'textarea',
      '#title' => '<span class="element-invisible">' . $title . '</span>',
      '#default_value' => variable_get('saveva_language_menu_path_list_' . $slm, ''),
      '#description' => $description,
    );

  }

  $fieldset = 'slm-fallback-settings';
  $form[$fieldset] = array(
    '#type' => 'fieldset',
    '#title' => t('Saveva language menu fallback handler settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

  $fallback_options = array('<nofallback>:0' => 'Disable fallback');
  for ($slm = 0; $slm < SAVEVA_LANGUAGE_MENUS; $slm++) {
    $check_menu = variable_get('saveva_language_menu_menu_' . $slm, '<nolangmenu>:0');
    if ($check_menu != '<nolangmenu>:0') {
      $menu_title = explode(':', $check_menu);
      $menu_one = menu_load($menu_title[0]);
      $title = $menu_one['title'];
      $fallback_options[$check_menu] = $title;
    }
  }

  $form[$fieldset]['saveva_language_menu_menu_fallback'] = array(
    '#type' => 'radios',
    '#title' => t('Fallback Menu'),
    '#description' => t('Select one of your configured menus for fallback handling. <em>(Best Guess: %guess)</em>', array('%guess' => '&lt;main-menu&gt;')),
    '#options' => $fallback_options,
    '#default_value' => variable_get('saveva_language_menu_menu_fallback', array()),
  );

  // Per-role fallback.
  $role_options = array_map('check_plain', user_roles());
  $adminrole = array_search('administrator', $role_options);
  if (FALSE !== $adminrole) {
    unset($role_options[$adminrole]);
  }
  $form[$fieldset]['saveva_language_menu_fallback_roles'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Force fallback for these roles'),
    '#default_value' => variable_get('saveva_language_menu_fallback_roles', array()),
    '#options' => $role_options,
    '#description' => t('Even if a page is available through links, buttons and hyperlink elements, users will be redirected if the page is not allowed within the selected menu configuration.'),
  );
  return saveva_language_menu_system_settings_form($form, FALSE);
}

/**
 * Custom submission handler for the settings form.
 */
function saveva_language_menu_admin_settings_submit($form, &$form_state) {
  $values = &$form_state['values'];
  $restricter = 0;
  $clear_all = TRUE;
  for ($slm = 0; $slm < SAVEVA_LANGUAGE_MENUS; $slm++) {
    $first_menu = explode(':', $values['saveva_language_menu_menu_' . $slm]);
    $selected_languages = $values['saveva_language_menu_languages_' . $slm];
    $path_list = $values['saveva_language_menu_path_list_' . $slm];
    $path_visibility = $values['saveva_language_menu_path_visibility_' . $slm];
    foreach ($selected_languages as $lang => $state) {
      if (!$state) {
        unset($selected_languages[$lang]);
      }
    }
    if (is_array($selected_languages) && count($selected_languages) > 1 && $first_menu[0] != '<nolangmenu>') {
      $clear_all = FALSE;
      $menu_one = menu_load($first_menu[0]);
      $title = $menu_one['title'];
      if ($restricter < 2) {
        $restricter++;
        // Make sure that we can translate and localize in the menu context
        // that's invasive but there's no other way to do this.
        $menu_one['i18n_mode'] = I18N_MODE_MULTIPLE;
        menu_save($menu_one);
        $menus = array($first_menu[0] => $title);
        $menu_one = menu_parent_options($menus, array('mlid' => 0));
        $menu_link = menu_link_load($first_menu[1]);
        if (($menu_link && $menu_link['module'] != 'saveva_language_menu') || !$menu_link) {
          $plid = $first_menu[1];
          if ($values['saveva_language_menu_title_' . $slm]) {
            $plid = saveva_language_menu_parent_menu_entry($first_menu, $values['saveva_language_menu_title_' . $slm], $slm, $path_visibility, $path_list);
            saveva_language_menu_lang_menu_entry($first_menu, $plid, $slm, $selected_languages, $values['saveva_language_menu_nativename_' . $slm], $path_visibility, $path_list);
          }
          else {
            saveva_language_menu_lang_menu_entry($first_menu, $plid, $slm, $selected_languages, $values['saveva_language_menu_nativename_' . $slm], $path_visibility, $path_list);
            // It is save to delete parent now.
            saveva_language_menu_parent_delete($first_menu, $slm);
          }
          drupal_set_message(t('Menu @menunumber: Please use the !menu for @menu. There is no need to adapt link paths. This is handled automatically. Translation is available using !translate or the translate link in !menu.',
           array(
             '@menu' => $title,
             '!translate' => l(t('String translation'),
             'admin/config/regional/translate/translate'),
             '!menu' => l(t('Menu management'), 'admin/structure/menu'),
             '@menunumber' => ($slm + 1),
           )));
        }
        else {
          drupal_set_message(t('Please do not use Saveva Language Menu items as parents. This @menu is forbidden.', array('@menu' => $title)), 'warning');
        }
      }
      else {
        // We never know... needs, wishes...
        $restricter = 0;
        drupal_set_message(t('You are not allowed to add more than two language sets per menu. Sorry!'), 'warning');
      }
    }
    else {
      $before = variable_get('saveva_language_menu_menu_' . $slm, '<nolangmenu>:0');
      if ($before != $values['saveva_language_menu_menu_' . $slm]) {
        saveva_language_menu_delete(explode(':', $before), $slm);
      }
    }
    if (is_array($selected_languages) && count($selected_languages) < 2 && $first_menu[0] != '<nolangmenu>') {
      saveva_language_menu_delete($first_menu, $slm);
    }
  }
  if ($clear_all) {
    saveva_language_menu_cleanup();
  }
}

/**
 * Remove all module related menu items for a given language set.
 */
function saveva_language_menu_cleanup() {
  $menu_task = menu_parent_options(menu_get_menus(), array('mlid' => 0));
  foreach ($menu_task as $menu_id => $menu_item) {
    $menu_fetch = explode(':', $menu_id);
    $menu_link = menu_link_load($menu_fetch[1]);
    if ($menu_link && $menu_link['module'] == 'saveva_language_menu') {
      menu_link_delete($menu_link['mlid']);
    }
  }
}

/**
 * Delete a specific menu item from menus for a given language set.
 */
function saveva_language_menu_delete($saveva_menu, $delta) {
  $menu_task = menu_load($saveva_menu[0]);
  $menus = array($saveva_menu[0] => $menu_task['title']);
  $menu_task = menu_parent_options($menus, array('mlid' => 0));

  foreach ($menu_task as $menu_id => $menu_item) {
    $menu_fetch = explode(':', $menu_id);
    $menu_link = menu_link_load($menu_fetch[1]);
    if ($menu_link['module'] == 'saveva_language_menu' && $menu_link['options']['saveva_delta'] == $delta) {
      menu_link_delete($menu_link['mlid']);
    }
  }
}

/**
 * Delete the parent of specified menu item from menus for a given language set.
 */
function saveva_language_menu_parent_delete($saveva_menu, $delta) {
  $menu_task = menu_load($saveva_menu[0]);
  $menus = array($saveva_menu[0] => $menu_task['title']);
  $menu_task = menu_parent_options($menus, array('mlid' => 0));
  $plid = 0;
  foreach ($menu_task as $menu_id => $menu_item) {
    $menu_fetch = explode(':', $menu_id);
    $menu_link = menu_link_load($menu_fetch[1]);
    if ($menu_link['module'] == 'saveva_language_menu' && $menu_link['options']['saveva_parent'] && $menu_link['options']['saveva_delta'] == $delta) {
      $plid = $menu_link['plid'];
      menu_link_delete($menu_link['mlid']);
    }
  }
  return $plid;
}

/**
 * Configure the parent of a given language set.
 */
function saveva_language_menu_parent_menu_entry($saveva_menu, $title, $delta, $path_visibility, $path_list) {
  $menu_task = menu_load($saveva_menu[0]);
  $menus = array($saveva_menu[0] => $menu_task['title']);
  $menu_task = menu_parent_options($menus, array('mlid' => 0));
  $item = FALSE;
  foreach ($menu_task as $menu_id => $menu_item) {
    $menu_fetch = explode(':', $menu_id);

    $menu_link = menu_link_load($menu_fetch[1]);
    if ($menu_link && $menu_link['module'] == 'saveva_language_menu' && $menu_link['options']['saveva_delta'] == $delta && $menu_link['options']['saveva_parent']) {
      $item = $menu_link;
      // Will be overwritten if necessary.
      $item['link_path'] = '<nolink>';
      $item['link_title'] = $title;
      $item['options']['alter'] = TRUE;
      $item['options']['saveva_parent'] = TRUE;
      $item['options']['saveva_delta'] = $delta;
      $item['options']['saveva_visibility'] = $path_visibility;
      $item['options']['saveva_pathlist'] = $path_list;
      $item['language'] = LANGUAGE_NONE;
      $item['hidden'] = 0;
      $item['expanded'] = 1;
      // Parent menu item, 0 if menu item is on top level.
      $item['plid'] = $saveva_menu[1];
      $item['customized'] = 1;
      $item['i18n_menu'] = TRUE;
    }
  }
  if (!$item) {
    // Will be overwritten if necessary.
    $item['link_path'] = '<nolink>';
    $item['link_title'] = $title;
    $item['menu_name'] = $saveva_menu[0];
    $item['weight'] = 1000;
    $item['module'] = 'saveva_language_menu';
    // Parent menu item, 0 if menu item is on top level.
    $item['plid'] = $saveva_menu[1];
    $item['language'] = LANGUAGE_NONE;
    $item['options']['alter'] = TRUE;
    $item['options']['saveva_parent'] = TRUE;
    $item['options']['saveva_delta'] = $delta;
    $item['options']['saveva_visibility'] = $path_visibility;
    $item['options']['saveva_pathlist'] = $path_list;
    $item['hidden'] = 0;
    $item['expanded'] = 1;
    $item['customized'] = 1;
    $item['i18n_menu'] = TRUE;
  }
  $mlid = menu_link_save($item);
  return $mlid;
}

/**
 * Configure a language menu item for a given language set.
 */
function saveva_language_menu_lang_menu_entry($saveva_menu, $plid, $delta, $selected_languages, $use_native, $path_visibility, $path_list) {
  $menu_task = menu_load($saveva_menu[0]);
  $menus = array($saveva_menu[0] => $menu_task['title']);
  $menu_task = menu_parent_options($menus, array('mlid' => 0));
  $existing_items = array();
  foreach ($menu_task as $menu_id => $menu_item) {
    $menu_fetch = explode(':', $menu_id);
    $menu_link = menu_link_load($menu_fetch[1]);
    if ($menu_link && $menu_link['module'] == 'saveva_language_menu' && $menu_link['options']['saveva_delta'] == $delta && !$menu_link['options']['saveva_parent']) {
      if (!in_array($menu_link['options']['saveva_lang'], $selected_languages)) {
        menu_link_delete($menu_link['mlid']);
      }
      else {
        $menu_link['plid'] = $plid;
        $existing_items[$menu_link['options']['saveva_lang']] = $menu_link;
      }
    }
  }

  $language_list = language_list();
  foreach ($language_list as $lang => $langobj) {
    if ($langobj->enabled && in_array($lang, $selected_languages)) {
      if (!$existing_items[$lang]) {
        $item = array();
        // Will be overwritten if necessary.
        $item['link_path'] = '<front>';
        $item['link_title'] = $use_native ? $langobj->native : $langobj->name;
        $item['menu_name'] = $saveva_menu[0];
        $item['module'] = 'saveva_language_menu';
        // Parent menu item, 0 if menu item is on top level.
        $item['plid'] = $plid;
        $item['options']['alter'] = TRUE;
        $item['options']['saveva_lang'] = $lang;
        $item['options']['saveva_native'] = $use_native;
        $item['options']['saveva_visibility'] = $path_visibility;
        $item['options']['saveva_pathlist'] = $path_list;
        $item['options']['saveva_delta'] = $delta;
        $item['options']['saveva_parent'] = FALSE;
        $item['hidden'] = 0;
        $item['customized'] = 1;
        $menu_link['i18n_menu'] = TRUE;
        $existing_items[$lang] = $item;
      }
      else {
        $existing_items[$lang]['link_title'] = $use_native ? $langobj->native : $langobj->name;
        $existing_items[$lang]['options']['saveva_native'] = $use_native;
        $existing_items[$lang]['options']['saveva_visibility'] = $path_visibility;
        $existing_items[$lang]['options']['saveva_pathlist'] = $path_list;
        $existing_items[$lang]['options']['saveva_delta'] = $delta;
        $existing_items[$lang]['options']['saveva_parent'] = FALSE;
        // Will be overwritten if necessary.
        $existing_items[$lang]['link_path'] = '<front>';
        $existing_items[$lang]['customized'] = 1;
        $existing_items[$lang]['i18n_menu'] = TRUE;
      }
      menu_link_save($existing_items[$lang]);
    }
  }
}

/**
 * Allow translation of language menu items.
 */
function saveva_language_menu_assure_menu_translation($translation, $menu) {
  require_once DRUPAL_ROOT . '/includes/locale.inc';
  $languages = language_list();
  $default_language = language_default('language');
  foreach ($languages as $lang => $langobj) {
    if ($lang != $default_language) {
      $check = i18n_string_translate('menu:menu:' . $menu . ':title', $translation, array('langcode' => $lang));
      if ($check == $translation) {
        i18n_string_translation_update('menu:menu:' . $menu . ':title', $translation, $lang);
      }
    }
  }
}

/**
 * Helper to clear the locale cache.
 */
function saveva_language_menu_locale_clear_cache() {
  require_once DRUPAL_ROOT . '/includes/locale.inc';
  // Clear locale cache.
  _locale_invalidate_js();
  cache_clear_all('locale:', 'cache', TRUE);
}
